/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Leo Mehr Holz
 *      Pascal Schärli
 *
 */

/**
    @file FlappyOrx.h
    @brief Gametype.
    @ingroup FlappyOrx
*/

#ifndef _FlappyOrx_H__
#define _FlappyOrx_H__

#include "flappyorx/FlappyOrxPrereqs.h"

#include "gametypes/Deathmatch.h"
#include <vector>

namespace orxonox
{
    struct Circle{
        int r;
        int x;
        int y;
        Circle(int new_r, int new_x, int new_y){
            r=new_r;
            x=new_x;
            y=new_y;
        }
        Circle(){
            r=0;
            x=0;
            y=0;
        }
    };

    class _FlappyOrxExport FlappyOrx : public Deathmatch
    {
        public:
            FlappyOrx(Context* context);

            virtual void start() override;
            virtual void end() override;
            virtual void death();
            virtual void addBots(unsigned int amount) override{} //<! overwrite function in order to bypass the addbots command
            virtual void XMLPort(Element& xmlelement, XMLPort::Mode mode);
            
            void setspawnDistance( int spawnDistance)
                {this->spawnDistance = spawnDistance; }
            int getspawnDistance()
                { return this->spawnDistance; }  
            inline void setSpeed( float speed ){   
                orxout()<< speed<< endl;
                this->speed = speed; }
            inline float getSpeed( )
                { return this->speed; }

            void updatePlayerPos(int x);
            void createAsteroid(Circle &c);
            void asteroidField(int x, int y, float slope);
            void spawnTube();
            
            void setCenterpoint(FlappyOrxCenterPoint* center);
           
            int getPoints(){return this->point;}
            
            void levelUp();
            
            bool isDead();
            void setDead(bool value);

            FlappyOrxShip* getPlayer();
            
            bool bEndGame;
            bool bIsDead;
            bool firstGame;
            std::string sDeathMessage;
            std::queue<int> tubes;
            std::queue<MovableEntity*> asteroids;
            int spawnDistance;
            int tubeOffsetX;
            float speed = 100;
            
        private:
             
            const static int nCircles = 6;
            int circlesUsed;
            Circle circles[nCircles];

            void clearCircles();
            bool circleCollision(Circle &c1, Circle &c2);
            int addIfPossible(Circle c);

            WeakPtr<FlappyOrxCenterPoint> center_;
            WeakPtr<FlappyOrxShip> player;

            int level;
            int point;
            bool b_combo;

            const int NUM_ASTEROIDS = 5;


           const std::string Asteroid5[5] = {"Asteroid3_1","Asteroid3_2","Asteroid3_3","Asteroid3_4","Asteroid3_5"};
           const std::string Asteroid10[5] = {"Asteroid6_1","Asteroid6_2","Asteroid6_3","Asteroid6_4","Asteroid6_5"};
           const std::string Asteroid15[5] = {"Asteroid9_1","Asteroid9_2","Asteroid9_3","Asteroid9_4","Asteroid9_5"};
           const std::string Asteroid20[5] = {"Asteroid12_1","Asteroid12_2","Asteroid12_3","Asteroid12_4","Asteroid12_5"};
           
            std::vector<std::string> DeathMessage10 = {
                "You should really try that again",
                "You can do better, can you?",
                "Hey, maybe you get a participation award, that's good isn't it?",
                "Congratulations, you get a medal, a wooden one",
                "That was flappin bad!",
                "Well, that was a waste of time"};
            std::vector<std::string> DeathMessage30 = {
                "Getting better!",
                "Training has paid off, huh?",
                "Good average!",
                "That was somehow enjoyable to watch",
                "Flappin average",
                "Getting closer to something",
                "That wasn't crap, not bad"};
            std::vector<std::string> DeathMessage50 = {
                "Flappin great",
                "Good job!",
                "Okay, we give you a shiny medal, not a golden one, tough",
                "Maybe you should do that professionally",
                "That was really good,!",
                "We are proud of you"};
            std::vector<std::string> DeathMessageover50 = {
                "You're flappin amazing",
                "Fucking great job",
                "Wow, we're really impressed",
                "We will honor you!",
                "Please do that again!",
                "Take that golden medal! You've earned it"};
            



    };
}

#endif /* _FlappyOrx_H__ */
